"use strict";
const _ = require("lodash");
const NerveObject = require("./object");
const NerveProperty = require("./lib/property");
class NerveActiveUser extends NerveObject {
    constructor(app, options) {
        super();
        this.fields = {};
        this.attr = {};
        this.isFetchedState = false;
        this.app = app;
        this.options = _.merge({}, options);
    }
    request() {
        let self = this, promises = Object.keys(this.options.fields).map((field) => {
            return this.options.fields[field] && _.isFunction(self[field]) ? self[field]() : Promise.resolve();
        });
        return Promise.all(promises)
            .then((results) => {
            results.forEach((item) => {
                if (item && !item.error) {
                    _.merge(this.attr, item);
                }
            });
            this.emit('fetched');
        });
    }
    get(key) {
        return NerveProperty.get(this.attr, key);
    }
    setSingle(key, value) {
        NerveProperty.set(this.attr, key, value);
        return this;
    }
    set(key, value) {
        let attrs;
        if (_.isObject(key)) {
            attrs = key;
            Object.keys(attrs).forEach((key) => this.setSingle(key, attrs[key]));
        }
        else {
            this.setSingle(key, value);
        }
        return this;
    }
    toJSON() {
        return this.attr;
    }
    isAuthorized() {
        return !!this.get('isAuthorized');
    }
    fetched(callback) {
        if (this.isFetchedState) {
            callback();
        }
        else {
            this.on('fetched', callback);
        }
    }
}
module.exports = NerveActiveUser;
//# sourceMappingURL=active-user.js.map