"use strict";
const _ = require("lodash");
const NerveModule = require("./module");
const url = require("url");
const request = require("request");
class NerveApi extends NerveModule {
    constructor(page, options) {
        super(page.getApp(), options);
        this.response = null;
        this.requests = [];
        this.responses = [];
        this.page = page;
        this.app = page.getApp();
        this.options = _.assign({}, options);
    }
    getDataSources() {
        return [];
    }
    getAllDataSources() {
        let parent = Object.getPrototypeOf(this.constructor).prototype, dataSources;
        if (parent && _.isFunction(parent.getAllDataSources)) {
            dataSources = parent.getAllDataSources().concat(this.getDataSources());
        }
        else {
            dataSources = this.getDataSources();
        }
        return dataSources;
    }
    fetch() {
        return new Promise((resolve, reject) => {
            let promises = this.getAllDataSources().map((item) => {
                let result;
                if (!_.isFunction(item.isEnable) || item.isEnable.call(this)) {
                    result = this.request(item);
                }
                else {
                    result = Promise.resolve({
                        response: {}
                    });
                }
                return result;
            });
            promises.push(this.getLocalesVars());
            Promise.all(promises)
                .then((responses) => {
                let result;
                this.responses = responses.map((item) => {
                    this.requests.push(item.request);
                    return item.response;
                });
                try {
                    result = this.adapter(this.responses);
                    if (result instanceof Promise) {
                        result
                            .then((adapted) => {
                            this.response = adapted;
                            resolve(this.response);
                        })
                            .catch((err) => {
                            reject({
                                error: err
                            });
                        });
                    }
                    else {
                        this.response = result;
                        resolve(this.response);
                    }
                }
                catch (err) {
                    reject({
                        error: err
                    });
                }
            })
                .catch((result) => {
                this.requests.push(result.request);
                reject(result.error);
            });
        });
    }
    request(options) {
        return new Promise((resolve) => {
            request({
                url: url.resolve(this.app.getCfg('apiHost'), options.url)
            }, function (error, response, body) {
                resolve(JSON.parse(body));
            });
        });
    }
    adapter(responses) {
        let result = {};
        if (_.isArray(responses)) {
            responses.forEach(function (item) {
                _.merge(result, item);
            });
        }
        return result;
    }
    getResponse() {
        return this.response || {};
    }
    getRequests() {
        return this.requests;
    }
    getPage() {
        return this.page;
    }
    static getResponseItemByIndex(api, index) {
        let allDataSources = this.prototype.getAllDataSources() || [], selfDataSources = this.prototype.getDataSources() || [];
        return api.responses ? (api.responses[allDataSources.length - selfDataSources.length + index] || {}) : {};
    }
}
module.exports = NerveApi;
//# sourceMappingURL=api.js.map