"use strict";
const _ = require("lodash");
const browser = require("browser-detect");
class NerveHelpers {
    static digitFormat(str) {
        return str.toString().replace(/(\s)+/g, '').replace(/(\d{1,3})(?=(?:\d{3})+$)/g, '$1 ');
    }
    static objectToCamelCase(object) {
        let obj = {};
        Object.keys(object).forEach((key) => {
            let k;
            if (key === 'ID') {
                k = key.toLowerCase();
            }
            else {
                k = key.charAt(0).toLowerCase() + key.substr(1);
            }
            obj[k] = object[key];
        });
        return obj;
    }
    static snakeToCamelCase(str) {
        let s = str;
        s = s.toLowerCase();
        if (s.length > 3) {
            s = s
                .replace(/_(.)/g, $1 => $1.toUpperCase())
                .replace(/_/g, '')
                .replace(/^(.)/, $1 => $1.toLowerCase());
        }
        return s;
    }
    static camelToSnakeCase(s) {
        return s
            .replace(/(?:^|\.?)([A-Z])/g, (x, y) => '_' + y.toLowerCase())
            .replace(/^_/, '');
    }
    static objectCamelToSnakCase(object) {
        const obj = {};
        Object.keys(object).forEach((key) => {
            obj[this.camelToSnakeCase(key)] = object[key];
        });
        return obj;
    }
    static objectSnakeToCamelCase(object) {
        const obj = {};
        Object.keys(object).forEach((key) => {
            obj[this.snakeToCamelCase(key)] = object[key];
        });
        return obj;
    }
    static secondToStr(s, format) {
        s = Number(s);
        if (isNaN(s)) {
            s = 0;
        }
        let timeFormat = _.assign({}, {
            showHour: false,
            showMin: true,
            showSec: true,
            padHour: false,
            padMin: true,
            padSec: true,
            sepHour: ':',
            sepMin: ':',
            sepSec: ''
        }, format), myTime = new Date(s * 1000), hour = myTime.getUTCHours(), min = timeFormat.showHour ? myTime.getUTCMinutes() : myTime.getUTCMinutes() + hour * 60, sec = timeFormat.showMin ? myTime.getUTCSeconds() : myTime.getUTCSeconds() + min * 60, strHour = (timeFormat.padHour && hour < 10) ? '0' + hour : hour, strMin = (timeFormat.padMin && min < 10) ? '0' + min : min, strSec = (timeFormat.padSec && sec < 10) ? '0' + sec : sec, strTime = '';
        if (strHour) {
            strTime += timeFormat.showHour ? strHour + timeFormat.sepHour : '';
        }
        strTime += timeFormat.showMin ? strMin + timeFormat.sepMin : '';
        strTime += timeFormat.showSec ? strSec + timeFormat.sepSec : '';
        return strTime;
    }
    static numberHumanFormat(value, separator) {
        let humanNumber;
        separator = separator || '';
        if (value < 1000) {
            humanNumber = String(value);
        }
        else if (value < 999950) {
            humanNumber = (value / 1000).toFixed(1).replace(/\.0$/, '') + separator + 'K';
        }
        else {
            humanNumber = (value / 1000000).toFixed(1).replace(/\.0$/, '') + separator + 'M';
        }
        return humanNumber;
    }
    static plural(value, text5, text1, text2) {
        let text;
        if (value < 0) {
            value = value * (-1);
        }
        value = value % 100;
        if (value >= 5 && value <= 14) {
            text = text5;
        }
        else {
            value = value % 10;
            if (!value || value >= 5) {
                text = text5;
            }
            else if (value >= 2) {
                text = text2;
            }
            else {
                text = text1;
            }
        }
        return text;
    }
    static capitalize(str) {
        return str.charAt(0).toUpperCase() + str.substr(1);
    }
    static cutText(text, length, options) {
        let cutted = text.slice(0, Number(length)), textLength = text.length, cuttedLength = cutted.length, posOpenLink, posCloseLink, i;
        options = options || {};
        options = _.merge(options, {
            maxLinkLength: 30
        });
        if (textLength > cuttedLength) {
            i = cuttedLength;
            while (text[i] !== ' ' && i < textLength) {
                cutted += text[i++];
            }
            posOpenLink = cutted.toLowerCase().lastIndexOf('<a');
            posCloseLink = cutted.toLowerCase().lastIndexOf('</a>');
            if (posOpenLink > posCloseLink) {
                cutted += text.slice(cutted.length, text.indexOf('</a>', cutted.length)) + '</a>';
            }
        }
        if (options.maxLinkLength) {
            cutted = cutted.replace(/>(.+?)<\/a>/g, function (matches, text) {
                let result = '>' + text.slice(0, options.maxLinkLength);
                if (text.length > options.maxLinkLength) {
                    result += '...';
                }
                result += '</a>';
                return result;
            });
        }
        return cutted;
    }
}
NerveHelpers.os = {
    isWindowsPhone: (userAgent) => {
        return /windows phone/i.test(userAgent);
    },
    isAndroid: (userAgent) => {
        return /android/i.test(userAgent);
    },
    isIOs: (userAgent) => {
        return /iPad|iPhone|iPod/.test(userAgent);
    },
    isMobile: (userAgent) => {
        return NerveHelpers.os.isWindowsPhone(userAgent) || NerveHelpers.os.isAndroid(userAgent) || NerveHelpers.os.isIOs(userAgent);
    }
};
NerveHelpers.browser = {
    getName: function (userAgent) {
        return browser(userAgent).name;
    },
    getVersion: function (userAgent) {
        return browser(userAgent).version;
    }
};
module.exports = NerveHelpers;
//# sourceMappingURL=helpers.js.map