"use strict";
const path = require("path");
const fs = require("fs");
const gettextParser = require("gettext-parser");
const debug = require("./debug");
let locales = {};
class NerveLocales {
    static init(app) {
        let localesDir = app.getCfg('localesDir');
        if (localesDir) {
            fs.readdir(localesDir, (err, files) => {
                if (err) {
                    debug.error('Failed read locales');
                }
                else {
                    files.forEach((locale) => {
                        let filePath = path.resolve(app.getCfg('localesDir'), locale, app.getCfg('localesFileName'));
                        fs.readFile(filePath, (err, content) => {
                            if (err) {
                                debug.error(`Failed read locales file: ${filePath}: `, err);
                            }
                            else {
                                locales[locale] = gettextParser.po.parse(content.toString()).translations;
                            }
                        });
                    });
                }
            });
        }
    }
    static getText(message, locale, ctx) {
        let result;
        ctx = ctx || '';
        result = locales[locale] && locales[locale][ctx] && locales[locale][ctx][message] && locales[locale][ctx][message].msgstr && locales[locale][ctx][message].msgstr[0];
        return result || message;
    }
}
module.exports = NerveLocales;
//# sourceMappingURL=locales.js.map