"use strict";
const _ = require("lodash");
const NerveObject = require("./object");
const fs = require("fs");
const locales = require("./lib/locales");
const debug = require("./lib/debug");
let localesCache = {};
class NerveModule extends NerveObject {
    constructor(app, options) {
        super();
        this.options = _.assign({}, options);
        this.app = app;
    }
    getApp() {
        return this.app;
    }
    getActiveUser() {
        return this.activeUser;
    }
    setActiveUser(activeUser) {
        this.activeUser = activeUser;
        return this;
    }
    getLocales() {
        return null;
    }
    readLocales(pathToFile) {
        let locales;
        return new Promise((resolve, reject) => {
            fs.readFile(pathToFile, (err, content) => {
                if (err) {
                    debug.error(err.toString());
                    reject(err);
                }
                else {
                    locales = JSON.parse(content.toString());
                    resolve(this.walkLocales(locales));
                }
            });
        });
    }
    walkLocales(locales) {
        let result = {};
        Object.keys(locales).forEach((key) => {
            let item = locales[key];
            if (_.isString(item)) {
                result[key] = this.getTextBySource(item);
            }
            else if (_.isObject(item) && item.text && (item.vars || item.ctx)) {
                result[key] = this.getTextBySource(item.text, item.ctx, item.vars);
            }
            else if (_.isObject(item)) {
                result[key] = this.walkLocales(item);
            }
        });
        return result;
    }
    getTextBySource(message, ctx, params) {
        let localeStr, globalParams = this.getLocalesParams();
        if (_.isObject(ctx)) {
            params = ctx;
            ctx = '';
        }
        localeStr = locales.getText(message, this.activeUser.get('locale'), ctx) || message;
        if (params || globalParams) {
            params = _.assign({}, params, globalParams);
            Object.keys(params).forEach(function (item) {
                var reg = new RegExp('##' + item + '##', 'g');
                localeStr = localeStr.replace(reg, params[item]);
            });
        }
        return localeStr;
    }
    getText(id) {
        let currentLocale = this.activeUser.get('locale'), localesObject = localesCache[this.constructor.name] && localesCache[this.constructor.name][currentLocale] ? localesCache[this.constructor.name][currentLocale] : {}, arIds = id.split('.'), iteration = 0, localesItem = localesObject;
        while (localesItem && iteration < arIds.length) {
            if (localesItem[arIds[iteration]]) {
                localesItem = localesItem[arIds[iteration]];
            }
            else {
                localesItem = null;
            }
            iteration++;
        }
        return localesItem;
    }
    getLocalesParams() {
        return null;
    }
    getLocalesVars() {
        let currentLocale = this.activeUser.get('locale'), localesPromise, localesObject = {};
        if (!localesCache[this.constructor.name] || (this.app && this.app.getCfg('isTestServer'))) {
            localesCache[this.constructor.name] = {};
        }
        return new Promise((resolve) => {
            if (localesCache[this.constructor.name][currentLocale]) {
                resolve({
                    locales: localesCache[this.constructor.name][currentLocale]
                });
            }
            else {
                localesPromise = this.getLocales();
                if (localesPromise) {
                    localesPromise
                        .then((locales) => {
                        if (Array.isArray(locales)) {
                            locales.forEach((localesItem) => localesObject = _.assign(localesObject, localesItem));
                        }
                        else {
                            localesObject = locales;
                        }
                        localesCache[this.constructor.name][currentLocale] = localesObject;
                        resolve({
                            locales: localesObject
                        });
                    });
                }
                else {
                    resolve({});
                }
            }
        });
    }
}
module.exports = NerveModule;
//# sourceMappingURL=module.js.map