"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const _ = require("lodash");
const url = require("url");
const path = require("path");
const debug = require("./lib/debug");
const NerveModule = require("./module");
const ActiveUser = require("./active-user");
class NervePage extends NerveModule {
    constructor(app, options) {
        super(app, options);
        this.baseTmplPath = '';
        this.pagesTmplPath = '';
        this.templateHead = '';
        this.template = '';
        this.templateFooter = '';
        this.templateError404 = '';
        this.templateError403 = '';
        this.templateError500 = '';
        this.storeState = {};
        this.isFirstContentRender = false;
        this.init();
        let templateHeadPath, templateFooterPath, templatePath;
        try {
            _.merge(this.options, {
                isNeedActiveUser: true,
                isShowErrorPage: true,
                isForceShowErrorPage: false,
                type: null,
                name: null
            }, options);
            this.startTime = Date.now();
            this.time('FULL PAGE TIME');
            this.log(`START`);
            this.log(`${this.getRequestMethod()} ${this.getLogUrl()}`);
            this.frontEndDir = this.getFrontendDir();
            if (!this.frontEndDir) {
                this.errorLog('FRONTEND DIR IS EMPTY, SET FRONTEND DIR TO CONFIG');
            }
            if (this.templateHead) {
                templateHeadPath = path.resolve(this.frontEndDir, this.baseTmplPath, this.templateHead);
                this.tmplHead = this.getTemplate(templateHeadPath);
            }
            if (this.templateFooter) {
                templateFooterPath = path.resolve(this.frontEndDir, this.baseTmplPath, this.templateFooter);
                this.tmplFooter = this.getTemplate(templateFooterPath);
            }
            if (this.template) {
                templatePath = path.resolve(this.frontEndDir, this.pagesTmplPath, this.template);
                this.tmpl = this.getTemplate(templatePath);
            }
            [403, 404, 500].forEach((errorCode) => {
                let templateErrorPath, self = this;
                if (self['templateError' + errorCode]) {
                    templateErrorPath = path.resolve(this.frontEndDir, this.baseTmplPath, self['templateError' + errorCode]);
                    self['tmplError' + errorCode] = this.getTemplate(templateErrorPath);
                }
            });
            this.initActiveUser();
            this.time('GET API RESPONSE');
            if (this.Api) {
                this.api = new this.Api(this, {
                    request: this.options.request,
                    response: this.options.response
                });
                this.api.setActiveUser(this.activeUser);
            }
            else {
                this.debug('API IS EMPTY');
            }
            this.getResponsePromises()
                .then((response) => {
                this.startProcessingTime = Date.now();
                this.timeEnd('GET API RESPONSE');
                this.time('PAGE PROCESSING');
                this.time('GET LOCALES');
                this.getLocalesVars()
                    .then((localesVars) => {
                    this.timeEnd('GET LOCALES');
                    this.time('TEMPLATE VARS');
                    this.getTemplateVars()
                        .then((vars) => {
                        this.timeEnd('TEMPLATE VARS');
                        vars = _.assign({
                            activeUser: {}
                        }, response, localesVars, vars);
                        _.merge(vars.activeUser, this.activeUser.toJSON());
                        this.sendResponse(vars);
                    })
                        .catch((err) => this.errorHandler(err));
                })
                    .catch((err) => this.errorHandler(err));
            })
                .catch((err) => this.errorHandler(err));
        }
        catch (err) {
            this.errorHandler(err);
        }
    }
    init() {
        return void (0);
    }
    getTemplate(templatePath) {
        let tmpl;
        if (this.app.getCfg('isClearTemplateCache') && require.cache) {
            delete require.cache[require.resolve(templatePath)];
        }
        tmpl = require(templatePath);
        return tmpl;
    }
    initActiveUser() {
        this.activeUser = new ActiveUser(this.app, {
            request: this.options.request,
            response: this.options.response
        });
    }
    getName() {
        return null;
    }
    getFrontendDir() {
        return this.app.getCfg('frontendDir');
    }
    getType() {
        return this.options.type;
    }
    getTitle() {
        return 'NerveJS Application';
    }
    getScheme() {
        return this.options.request.protocol;
    }
    getRequestParam(paramName) {
        return this.options.request.params[paramName];
    }
    getStaticHost() {
        return '//' + this.app.getCfg('hosts.static');
    }
    getJsHost() {
        return '//' + this.app.getCfg('hosts.js');
    }
    getCssHost() {
        return '//' + this.app.getCfg('hosts.css');
    }
    getCssVersion(cssName) {
        return this.app.getCfg('isUseCssHash') ? (this.app.getCssVersions()[cssName] || cssName) : cssName;
    }
    getJsVersion(jsName) {
        return this.app.getCfg('isUseJsHash') ? (this.app.getJsVersions()[jsName] || jsName) : jsName;
    }
    getCssUrl(cssName) {
        let cssUrl = url.resolve(this.getCssHost(), this.getCssVersion(cssName));
        if (!/\.css$/.test(cssUrl)) {
            cssUrl += '.css';
        }
        return cssUrl;
    }
    getJsUrl(jsName) {
        let jsUrl;
        if (this.app.getCfg('isUseJsMin')) {
            jsUrl = url.resolve(this.getJsHost(), 'min/') + this.getJsVersion(jsName) + '.js';
        }
        else {
            jsUrl = url.resolve(this.getJsHost(), this.getJsVersion(jsName) + '.js');
        }
        return jsUrl;
    }
    getSprite(sprite) {
        return this.app.getSpriteVersions()[sprite];
    }
    getCss() {
        return [];
    }
    getJs() {
        return [];
    }
    getResponsePromises() {
        return new Promise((resolve, reject) => {
            new Promise((userResolve, userReject) => {
                if (this.options.isNeedActiveUser) {
                    this.activeUser.request()
                        .then(userResolve)
                        .catch(userReject);
                }
                else {
                    userResolve();
                }
            })
                .then(() => {
                let resultBeforeFilter = this.beforeFilter() || {};
                if (resultBeforeFilter instanceof Promise) {
                    resultBeforeFilter
                        .then((result) => {
                        if (!result.isAbort) {
                            this.fetchApi()
                                .then(resolve)
                                .catch(reject);
                        }
                    })
                        .catch(reject);
                }
                else {
                    if (!resultBeforeFilter.isAbort) {
                        this.fetchApi()
                            .then(resolve)
                            .catch(reject);
                    }
                }
            })
                .catch(reject);
        });
    }
    getTemplateVars() {
        return new Promise((resolve, reject) => {
            this.getLocalesVars()
                .then(() => {
                resolve({
                    request: {
                        url: this.getRequestUrl(),
                        path: this.options.request.path,
                        get: this.options.request.query,
                        params: this.options.request.params,
                        query: this.options.request.query,
                        cookies: this.options.request.cookies,
                    },
                    css: this.getCss(),
                    js: this.getJs(),
                    hosts: {
                        static: this.getStaticHost(),
                        staticJs: this.getJsHost(),
                        staticCss: this.getCssHost()
                    },
                    pageTitle: this.getTitle(),
                    routes: this.app.getPublicRoutes()
                });
            })
                .catch(reject);
        });
    }
    getTemplateHeadVars() {
        return __awaiter(this, void 0, void 0, function* () {
            return {};
        });
    }
    getTemplateFooterVars() {
        return __awaiter(this, void 0, void 0, function* () {
            return {};
        });
    }
    getErrorTemplateVars() {
        return NervePage.prototype.getTemplateVars.call(this);
    }
    getLogPrefix() {
        let date = (new Date()).toString(), pageName = this.getName(), user = this.activeUser && this.activeUser.get('email') ? this.activeUser.get('email') : 'unauthorized';
        return `[${process.pid}] ${this.getRequestId()} ${date}: ${pageName}: ${user}`;
    }
    getRequestId() {
        return this.options.request && this.options.request.id ? this.options.request.id : null;
    }
    getRequestMethod() {
        return this.options.request && this.options.request.method ? this.options.request.method : null;
    }
    getRequestUrl() {
        return this.options.request && this.options.request.url ? this.options.request.url : null;
    }
    getLogUrl() {
        return this.getRequestUrl();
    }
    fetchApi() {
        return new Promise((resolve, reject) => {
            if (this.api) {
                this.api.fetch()
                    .then(resolve)
                    .catch(reject);
            }
            else {
                resolve();
            }
        });
    }
    beforeFilter() {
        return null;
    }
    errorLog(message) {
        debug.error(`${this.getLogPrefix()}: ${message}`);
    }
    log(message) {
        if (!this.isPingPage() || this.app.getCfg('isLoggedPingRequests')) {
            debug.log(`${this.getLogPrefix()}: ${message}`);
        }
    }
    debug(message) {
        if (!this.isPingPage() || this.app.getCfg('isLoggedPingRequests')) {
            debug.debug(`${this.getLogPrefix()}: ${message}`);
        }
    }
    time(message) {
        if (!this.isPingPage() || this.app.getCfg('isLoggedPingRequests')) {
            debug.time(this.getRequestId() + message);
        }
    }
    timeEnd(message) {
        if (!this.isPingPage() || this.app.getCfg('isLoggedPingRequests')) {
            debug.timeEnd(this.getRequestId() + message, `${this.getLogPrefix()}: ${message}`);
        }
    }
    renderTemplate(template, vars, templateId) {
        return __awaiter(this, void 0, void 0, function* () {
            let html = '';
            if (template) {
                this.time(`RENDER ${templateId}`);
                if (template.default) {
                    template = template.default;
                }
                const result = template(vars);
                if (typeof (result) === 'string') {
                    html = result;
                }
                else if (result.then) {
                    const renderResult = yield result;
                    html = renderResult.html;
                    this.storeState = renderResult.store;
                }
                else {
                    this.debug('Template type unknown - returning empty string');
                }
                this.timeEnd(`RENDER ${templateId}`);
            }
            else {
                this.debug(`EMPTY ${templateId} TEMPLATE`);
            }
            return html;
        });
    }
    getHtml(vars, contentTmpl) {
        return __awaiter(this, void 0, void 0, function* () {
            this.time('RENDER');
            let head, content, footer;
            if (this.isFirstContentRender) {
                content = yield this.renderTemplate(_.isFunction(contentTmpl) ? contentTmpl : this.tmpl, vars, 'CONTENT');
                const headVars = yield this.getTemplateHeadVars();
                const footerVars = yield this.getTemplateFooterVars();
                [head, footer] = yield Promise.all([
                    this.renderTemplate(this.tmplHead, Object.assign(Object.assign({}, vars), headVars), 'HEAD'),
                    this.renderTemplate(this.tmplFooter, Object.assign(Object.assign({}, vars), footerVars), 'FOOTER'),
                ]);
            }
            else {
                [head, content, footer] = yield Promise.all([
                    this.renderTemplate(this.tmplHead, vars, 'HEAD'),
                    this.renderTemplate(_.isFunction(contentTmpl) ? contentTmpl : this.tmpl, vars, 'CONTENT'),
                    this.renderTemplate(this.tmplFooter, vars, 'FOOTER'),
                ]);
            }
            this.timeEnd('RENDER');
            return head + content + footer;
        });
    }
    getUserAgent() {
        return this.options.request && _.isFunction(this.options.request.get) ? this.options.request.get('User-Agent') || '' : '';
    }
    getReferrer() {
        return this.options.request.headers.referer;
    }
    getContentType() {
        return this.isJsonAccept() ? 'application/json' : 'text/html';
    }
    errorHandler(err) {
        let statusCode = err && err.statusCode || 500;
        if (err && err.name && err.name === 'UpstreamResponseError') {
            this.api.getRequests().forEach((request) => {
                if (request.getStatusCode() !== '200') {
                    statusCode = request.getStatusCode();
                    this.errorLog(`${err.name} ${request.getStatusCode()} ${request.getUpstream()} (${request.getHostname()}:${request.getPort()}) ${request.getMethod()} ${request.getPath()}`);
                }
            });
        }
        else {
            this.errorLog(`${__filename} : ${err && err.stack ? err + err.stack : err}`);
        }
        this.options.response.status(statusCode);
        this.httpStatus = statusCode;
        if (this.isJsonAccept()) {
            this.getLocalesVars()
                .then((localesVars) => {
                this.getErrorTemplateVars()
                    .then((vars) => {
                    this.send(JSON.stringify(_.assign({}, vars, localesVars, {
                        activeUser: this.activeUser.toJSON(),
                        statusCode: statusCode
                    })), null, 200);
                })
                    .catch((err) => {
                    this.errorLog(err.toString());
                    this.send('');
                });
            })
                .catch((err) => {
                this.errorLog(err);
                this.send('');
            });
        }
        else {
            if (this.app.getCfg('isTestServer') && !this.isForceShowErrorPage()) {
                this.send(err + '<br/>' + err ? err.stack.replace(/\n/g, '<br/>') : '', 'text/html');
            }
            else {
                if (this.isShowErrorPage()) {
                    this.renderErrorPage(statusCode);
                }
                else {
                    this.send('');
                }
            }
        }
        this.errorLog(`Error ${this.httpStatus} ${this.getRequestMethod()} ${this.getLogUrl()}`);
        return this;
    }
    isPingPage() {
        return false;
    }
    isShowErrorPage() {
        return !!this.options.isShowErrorPage;
    }
    isForceShowErrorPage() {
        return !!this.options.isForceShowErrorPage;
    }
    isJsonAccept() {
        return this.options.request.headers && this.options.request.headers.accept && this.options.request.headers.accept.indexOf('application/json') !== -1;
    }
    renderErrorPage(status) {
        let self = this;
        if (self['tmplError' + status]) {
            this.getLocalesVars()
                .then((localesVars) => {
                this.getErrorTemplateVars()
                    .then((vars) => {
                    this.getHtml(_.assign({}, vars, localesVars, {
                        activeUser: this.activeUser.toJSON(),
                        statusCode: status
                    }), self['tmplError' + status])
                        .then((html) => this.send(html))
                        .catch((err) => this.send(''));
                })
                    .catch((err) => {
                    this.errorLog(`${err.toString()} ${err.stack}`);
                    this.send('');
                });
            })
                .catch((err) => {
                this.errorLog(err.toString());
                this.send('');
            });
        }
        else {
            this.send('');
        }
    }
    redirect(location, status) {
        if (this.isJsonAccept()) {
            this.httpStatus = 200;
            this.send(JSON.stringify({
                isRedirect: true,
                location: location
            }));
        }
        else {
            this.httpStatus = status || 301;
            this.options.response.header({
                location: location
            });
            this.send();
        }
    }
    rewrite(url) {
        this.app.getRouter().go(url, this.options.request, this.options.response);
    }
    sendResponse(vars) {
        if (this.isJsonAccept()) {
            this.send(JSON.stringify(vars));
        }
        else {
            this.getHtml(vars)
                .then((html) => this.send(html))
                .catch((err) => this.errorHandler(err));
        }
    }
    send(content, contentType, status) {
        content = String(content || '');
        this.options.response.status(status || this.httpStatus || 200);
        this.options.response.header({
            'Cache-Control': 'no-cache, no-store',
            'Content-type': (contentType || this.getContentType()) + '; charset=utf-8'
        });
        this.timeEnd('PAGE PROCESSING');
        this.timeEnd('FULL PAGE TIME');
        this.log('END');
        this.processingTime = Date.now() - this.startProcessingTime;
        this.fullTime = Date.now() - this.startTime;
        this.options.response.send(content);
        this.emit('send', {
            text: content,
            status: this.httpStatus || 200
        });
    }
}
module.exports = NervePage;
//# sourceMappingURL=page.js.map